function [Xest, t, Outputs] = ACal(Y, M, param)
% Minimizes ||X||_1    s.t.   diag(t)*Y = M*X , 
%                             sum(t) = param.c
%
%
% param.rho : ADMM parameter for diag(t)*Y=M*X
% param.N : signal size
% param.M : measurement size
% param.L : number of experiments
% param.Nit : number of iterations
% param.stop_crit : the limit of error between 
%                           consecutive updates for convergence
% param.timelimit : the limit for execution time in minutes, when reached
%                   simualtion is paused and all data is saved in the param.resumefile
% param.resume : flag to indicate if an earlier simulation will be resumed
% param.resumefile : the file to load/save the data to resume/pause the
%                    simulation
tic;
% Parameters
[nb_meas, L] = size(Y); N = size(M, 2);
Outputs.toberesumed=0;

% Useful functions
soft = @(x, T) sign(x).*max(abs(x)-T, 0);

% Initializations
t = ones(nb_meas, 1);
Xest = zeros(N, L);
Lambda = zeros(nb_meas, L);
mu = 1/(param.rho*norm(M)^2);

% resume operation preperation, load from resumefile if paused before
iter_first = 1;
if(any(strcmp(fieldnames(param), 'resume')) && param.resume==1)
    max_iter=param.max_iter;
    tlimit=param.tlimit; % if time limit or number of iterations are now different use new values
    load(param.resumefile);
    param.max_iter=max_iter;
    param.tlimit=tlimit;
    clear tlimit max_iter;
end

iter=iter_first;

% Main iteration
while iter <= param.max_iter
    % Keep in memory previous estimate for stopping criterion
    prev = Xest;
    % Soft-thresholding
    Xest = soft(Xest - mu*M'*(Lambda + param.rho*(M*Xest - diag(t)*Y)), mu);
    % Calibration
    U = M*Xest; 
    if param.calibrate
        a = diag((Lambda + param.rho*U)*Y'); b = param.rho*diag(Y*Y');
        gamma = (nb_meas-sum(a./b))/(sum(1./b));
        t = (a+gamma)./b;
    end
    % Lagrangian
    Lambda = Lambda + param.rho*(U - diag(t)*Y);
    % Print log or not
    rel_change = norm(prev(:)-Xest(:))/norm(Xest(:));
    if (param.verbose && mod(iter,10)==0)
        fprintf(['%i, ||X||_1 = %e, ||MX-DY||_2 = %e, Tr(D) = %e, ', ...
            'stop_crit = %e\n'], ...
            iter, norm(Xest(:), 1), norm(M*Xest - diag(t)*Y, 'fro'), ...
            sum(t), rel_change);
    end
    
    % Stop if time limit is reached and save state to resumefile
    if ((toc/60)>(param.tlimit*0.95))
        iter_first = iter;
        save(param.resumefile);
        Outputs.toberesumed=1;
        disp(['Saving state to ' param.resumefile ' to resume later']);
        break;
    end
    
    % Stopping criterion
    if (rel_change<param.stop_crit)
        break;
    end
    
    iter = iter + 1;
end


end
