function [X, t, iter, Outputs] = CCal(g, M, param)
% Minimizes ||X||_1    s.t.   diag(t)*g = A * X(:) , X is positive semidefinite
%                             sum(t) = param.c
%
% ADMM formulation:
% Ind_pos.sem.def(V) + ||U||_1    s.t.    diag(t)*g = A * X(:) , V = X 
%                                         U = X, sum(t) = param.c
% X : N*LxN*L array
% g : MxLxL array
%
% Ind_pos.sem.def() is the indicator function for hermitian positive semi
% definite matrices
%
% param.mu : ADMM parameter for V=X, U=X and diag(t)*g=A * X(:)
% param.N : signal size
% param.M : measurement size
% param.L : number of experiments
% param.Nit : number of iterations
% param.convergence_err : the limit of error between 
%                           consecutive updates for convergence
% param.timelimit : the limit for execution time in minutes, when reached
%                   simualtion is paused and all data is saved in the param.resumefile
% param.resume : flag to indicate if an earlier simulation will be resumed
% param.resumefile : the file to load/save the data to resume/pause the
%                    simulation
tic;

% --- Useful functions

Nit = param.max_iter;
timelimit=param.timelimit;
c=size(g,1);
if(param.calib)
c=param.c;
end

k_first = 1;
if(any(strcmp(fieldnames(param), 'resume')) && param.resume==1)
    Nit2=Nit;
    timelimit2=timelimit;
    load(param.resumefile);
    Nit=Nit2;
    timelimit=timelimit2;    
    clear Nit2 timelimit2;
else
    NL = param.N*param.L; NL2 = NL^2;
    A = @(X) forward_op(X, param.N, param.L, M);
    At = @(y) adjoint_op(y, param.N, param.L, M);
    AtoA = @(X) adjoint_o_forward(X, param.N, param.L, M);
    Afun_eig = @(x) reshape(AtoA(reshape(x, NL, NL)), NL2, 1);

    opts.isreal = false;
    M_scale = real(eigs(Afun_eig,NL2,1,'LM',opts));
    mu1 = param.mu;
    mu2 = param.mu;
    mu3 = param.mu;
    Afun = @(x) reshape(mu3*AtoA(reshape(x, NL, NL)), NL2, 1) + (mu1+mu2)*x;

    convergence_err=param.convergence_err;

    show_corr =0;
    if(any(strcmp(fieldnames(param), 'x_org')))
        x_org = param.x_org;
        show_corr =1;
    end

    J_it = zeros(Nit, 5);
    X = At(g)/M_scale;
    
    D1 = zeros(size(X));
    D2 = zeros(size(X));
    d3 = zeros(size(g));
    V = zeros(size(X));
    U = zeros(size(X));
    t_0 = c*ones(size(g,1),1)/size(g,1);
    t = t_0;
    t_all = reshape(t*ones(1,param.L^2),size(g));
    
    zsq_sum = sum(sum(abs(g).^2,2),3);    
    G = null(ones(1,size(g,1)));
    GHinvGc = G*((G'*diag(zsq_sum)*G)\G');
    
    
    update_err = inf;

    corr_reached=false;
end

iter=k_first;
while (((iter<=Nit)&&(update_err>convergence_err)&&(~corr_reached)&&((toc/60)<(timelimit*0.95))))
 
     % V = mu1/2 || X - V - D1 ||_2^2 s.t. V >= 0
    %   =  1/2 (X - D1 - X' + D1') - 1/mu1 I        
    V = (X - D1 + X' - D1')/2;
    [C,S]=eig(V);
    S = real(diag(S)); ind_Snz = (S>1e-14);
    S(~ind_Snz)=0;
    V = C(:,ind_Snz)*diag(S(ind_Snz))*C(:,ind_Snz)';
    num_eigs = sum(ind_Snz);
    
    % U = argmin  || U ||_1 + mu2/2 || X - U - D2 ||_2^2
    %   = soft(X-D2 , 1/mu2)
    U = soft(X - D2, 1/mu2);
    
    % X = argmin mu1/2 || X - V - D1 ||_2^2 + mu2/2 || X - U - D2 ||_2^2 
    %            + mu3/2 || g - A * X(:) - d3 ||_2^2
    %   = ( (mu1+mu2) I + mu3 A'*A )^-1 [ mu3 A' *(g-d3) + mu1( V+D1 ) + mu2( U+D2 )]
    X_old = X;
    a = mu1*(V+D1) + mu2*(U+D2) + mu3*(At((t_all.*g-d3)));
    
    [m, fl] = pcg(Afun, a(:), 10^-8, 1000, [], [], X(:));
    X = reshape(m, size(X));
    if(fl)
        disp('Inversion of I + A"*A not converged');
    end
    update_err = norm(X(:)-X_old(:))/norm(X_old(:));
    AX = A(X);
    
    %update t
    if(param.calib)
        t = t_0 + GHinvGc*( sum(sum( conj(g).*(AX+d3) ,2),3) - c*zsq_sum/size(g,1) );   
        t_all = reshape(t*ones(1,param.L^2),size(g));
    end
    
    
    % Update D1, D2, d3
    D1 = D1 - (X-V);
    D2 = D2 - (X-U);
    d3 = d3 - (t_all.*g - AX);
    
    J_it(iter, 1) = sum(real(trace(V)));
    J_it(iter, 2) = sum(abs(U(:)));
    J_it(iter, 3) = sum(abs(X(:) - V(:)).^2);  
    J_it(iter, 4) = sum(abs(X(:) - U(:)).^2);  
    J_it(iter, 5) = sum(abs(t_all(:).*g(:) - AX(:)).^2);
    J_it(iter, 6) = update_err;
    
    if(mod(iter,10)==0)
        if(show_corr)
            [xrec, lbd] = eigs(X(:,:),1);
                xrec = xrec(:)*sqrt(lbd);
                x_corr = abs(sum(xrec(:)'*x_org(:)))/norm(xrec(:))/norm(x_org(:));
                corr_reached = x_corr>0.9999;
                disp(['It=' num2str(iter) ', Corr=' num2str(x_corr,'%e') ', Diff=' num2str(update_err)...
              ', V_rank=' num2str(num_eigs) ', Err1=' num2str(J_it(iter, 3))...
              ', Err2=' num2str(J_it(iter, 4)) ', Err3=' num2str(J_it(iter, 5))]);
        else
            
                disp(['It=' num2str(iter) ', Diff=' num2str(update_err)...
              ', V_rank=' num2str(num_eigs) ', Err1=' num2str(J_it(iter, 3))...
              ', Err2=' num2str(J_it(iter, 4)) ', Err3=' num2str(J_it(iter, 5))]);
        end
    end
    
    iter=iter+1;
end

if(show_corr)
    [xrec, lbd] = eigs(X(:,:),1);
        xrec = xrec(:)*sqrt(lbd);
        x_corr = abs(sum(xrec(:)'*x_org(:)))/norm(xrec(:))/norm(x_org(:));
        
        disp(['It=' num2str(iter-1) ', Corr=' num2str(x_corr) ', Diff=' num2str(update_err)...
              ', V_rank=' num2str(num_eigs) ', Err1=' num2str(J_it(iter-1, 3))...
              ', Err2=' num2str(J_it(iter-1, 4)) ', Err3=' num2str(J_it(iter-1, 5))]);
        else
        disp(['It=' num2str(iter-1) ', Diff=' num2str(update_err)...
              ', V_rank=' num2str(num_eigs) ', Err1=' num2str(J_it(iter-1, 3))...
              ', Err2=' num2str(J_it(iter-1, 4)) ', Err3=' num2str(J_it(iter-1, 5))]);
end

Outputs.toberesumed=0;
if(any(strcmp(fieldnames(param), 'resumefile')) && ((iter<=Nit)&&(update_err>convergence_err)&&(~corr_reached)))
    k_first = iter; %#ok<NASGU>
    save(param.resumefile);
    Outputs.toberesumed=1;
    disp(['Saving state to ' param.resumefile ' to resume later']);
end

Outputs.V = V;
Outputs.U = U;
Outputs.J_it = J_it;
end

function y = soft(x,T)
%% SOFT THRESHOLD
% y = soft(x,T)
% x : input data
% T : threshold

y = max(1-T./(abs(x)+eps),0) .* x;
end

function y = forward_op(X, N, L, M)

m = size(M,1);
y = zeros(m,L,L);
for el1=L:-1:1
    for el2=L:-1:1
    y(:,el1,el2) = sum((M*X( ((el1-1)*N+1):((el1-1)*N+N) , ((el2-1)*N+1):((el2-1)*N+N) )).*conj(M),2).';
    end
end

end

function X = adjoint_op(y, N, L, M)

X = zeros(N*L,N*L);
for el1=size(y,3):-1:1
    for el2=size(y,3):-1:1
    X( ((el1-1)*N+1):((el1-1)*N+N) , ((el2-1)*N+1):((el2-1)*N+N) ) = M'*diag(y(:,el1,el2))*M;
    end
end

end

function X = adjoint_o_forward(X, N, L, M)

%
for el1=L:-1:1
    for el2=L:-1:1
        y = sum((M*X( ((el1-1)*N+1):((el1-1)*N+N) , ((el2-1)*N+1):((el2-1)*N+N) )).*conj(M),2).';
        temp = M'*diag(y)*M;
        X( ((el1-1)*N+1):((el1-1)*N+N) , ((el2-1)*N+1):((el2-1)*N+N) ) = temp;
    end
end
end