function [recovery, lambda_up, lambda_low] = PCalLambda(Xtrue, M, param)
    recovery = 0;
    lambda_low = 0;
    lambda_up = inf;
    X = Xtrue(:)*Xtrue(:)';
    
    % The optimization for c=0 presents some unstabilities with the ADMM
    % implementation, therefore we limit the number of iterations.
    Nit = param.max_iter;
    param.max_iter = min(5000,Nit);
    fprintf('Optimizing for G_0\n');
    [~, G_Z0] = MinimizeG(X, M, 0, param);
    if(G_Z0 <=0)
        fprintf('Recovery not possible\n');
        return;
    end
    param.max_iter = Nit;    
    
    fprintf('Optimizing for G_-1\n');
    [~, G_Zm1] = MinimizeG(X, M, -1, param);
    if(G_Zm1 <=0)
        fprintf('Recovery not possible\n');
        return;
    else
        lambda_low = 1/G_Zm1;
    end
   
    fprintf('Optimizing for G_1\n');
    [~, G_Z1] = MinimizeG(X, M, 1, param);
    if(G_Z1 < 0)
        lambda_up = -1/G_Z1;
    end
    
    if lambda_low < lambda_up
        recovery = 1;
    else        
        fprintf('Recovery not possible\n');
    end
    
end

function [Z0, G_Z0] = MinimizeG(X, M, c, param)
% This is the optimization function used to find the range of lambda values
% for successfull recovery of X via CPRL (or phase calibration)
%
% minimizes     Real{ <sign(X), Z_Omega0> } + || Z_Omega0perp ||_1
% subject to    Trace( Z ) = c
%               sum((M'*Z).*(M.'), 2) = 0
%               Z_tilda(1,1) is real valued
%               Z_tilda(2:N,2:N) >= 0 (positive semi-definite)
%               Z_tilda = E'*Z*E
%
% where         Z_Omega0 is Z.*(X==0)
%               Z_Omega0perp is Z.*(X~=0)
%               E is the set of eigenvectors of X, E(:,1) being the
%               eigenvector for the non-zero eigenvalue
%
% Outputs: Z0:   final optimization result
%          G_Z0: final objective function value
%


% --- Useful functions
%A = @(x) sum((M'*x).*(M.'), 2);
%At = @(y) M*diag(y)*M';
%Afun = @(x) reshape(At(A(reshape(x, param.N, param.N))), [], 1) + x;
M = M';
Afun = @(x) reshape( At(A(reshape(x, param.N*param.L, param.N*param.L),M),M), [], 1) + x;
soft = @(x, T) sign(x).*max(abs(x)-T, 0);

% --- Initializations
% First group of variables
Z0 = zeros(param.N*param.L, param.N*param.L);
Z = zeros(param.N*param.L, param.N*param.L, 3); Z(1, 1, 2) = c;
n = sqrt(3)*param.N*param.L; % For stopping criterion
% Lagrange multipliers
L1 = zeros(param.N*param.L, param.N*param.L, 3);
L2 = zeros(size(M, 2)*(param.L)^2, 1);
AtL2 = zeros(size(Z0));
p = sqrt(3*(param.N*param.L)^2 + size(M, 2)*(param.L)^2); % For stopping criterion
% Other
[V, D] = eig(X); S = abs(diag(D)); % Eigenvectors
I = (S==max(S)); E = V; E(:, 1) = V(:, I); E(:, I) = V(:, 1); % Re-order
Omega = (X(:)==0);
signX = sign(X(not(Omega)));
% to track the behaviour of objective function for early stop
obj_func_incr_count = 0; 
obj_func_decr_count = 0;
G_Z0 = 0;

eps_abs = param.eps_abs;
eps_rel = param.eps_rel;
pcg_prec = param.pcg_prec;

% --- ADMM
for iter = 1:param.max_iter
    
    % --- Update first group of variables
    % Update Z1
    temp = Z0 - L1(:, :, 1)/param.rho;
    temp(not(Omega)) = temp(not(Omega)) - signX/param.rho;
    temp(Omega) = soft(temp(Omega), 1/param.rho);
    Z(:, :, 1) = temp;
    % Update Z2
    U = Z0 - L1(:, :, 2)/param.rho;
    U = E'*U*E;
    a = real(U(1));
    li = U(1, 2:end); co = U(2:end, 1); b = (conj(li(:))+co(:))/2;
    C = U(2:end, 2:end);
    [Cvec, Ceig] = eig((C+C')/2); Ceig(real(Ceig)<0) = 0; C = Cvec*real(Ceig)*Cvec';
    Z(:, :, 2) = E*[a b';b C]*E';
    % Update Z3
    % If close 
    if iter==1 
        ref = Z0 - (L1(:, :, 3) + AtL2)/param.rho;
        [temp, fl] = pcg(Afun, ref(:), 10^-10, 1000, [], [], Z0(:));
        if fl, disp('PCG did not converge'); end
        Z(:, :, 3) = reshape(temp, param.N*param.L, param.N*param.L);
    else
        ref = Z0 - (L1(:, :, 3) + AtL2)/param.rho;
        [temp, fl] = pcg(Afun, ref(:), pcg_prec, 1000, [], [], Z0(:));
        if fl, disp('PCG did not converge'); end
        Z(:, :, 3) = reshape(temp, param.N*param.L, param.N*param.L);
    end
    AZ3 = A(Z(:, :, 3),M);
    
    % --- Update second group of variables 
    % Keep old value in memory
    ZOld = Z0;
    % Update Z
    temp = (sum(Z, 3) + sum(L1, 3)/param.rho)/3;
    alpha = (sum(diag(temp)) - c)*param.rho/(param.N*param.L);
    temp(eye(param.N*param.L)==1) = temp(eye(param.N*param.L)==1) - alpha/param.rho;
    Z0 = temp;
    

    % Lagrangian
    L1(:, :, 1) = L1(:, :, 1) + param.rho*(Z(:, :, 1) - Z0);
    L1(:, :, 2) = L1(:, :, 2) + param.rho*(Z(:, :, 2) - Z0);
    L1(:, :, 3) = L1(:, :, 3) + param.rho*(Z(:, :, 3) - Z0);
    L2 = L2 + param.rho*(AZ3);
    AtL2 = At(L2,M);
    
    % Primal residual
    res_primal = sqrt(norm(Z(:, :, 1) - Z0, 'fro')^2 + ...
        norm(Z(:, :, 2) - Z0, 'fro')^2 + ...
        norm(Z(:, :, 3) - Z0, 'fro')^2 + ...
        norm(AZ3, 'fro')^2);
    res_dual = sqrt(3)*param.rho*norm(Z0-ZOld, 'fro');
    
    % Stopping variables
    Ax = sqrt(norm(Z(:))^2 + norm(AZ3, 'fro')^2);
    Bz = sqrt(3)*norm(Z0(:));
    Aty = sqrt(norm(L1(:, :, 1), 'fro')^2 + norm(L1(:, :, 2), 'fro')^2 + norm(L1(:, :, 3) + AtL2, 'fro')^2);
    G_Z0_old = G_Z0;
    G_Z0 = norm(Z0(Omega), 1) + real(signX'*Z0(not(Omega))); 
    if G_Z0 < G_Z0_old && G_Z0<0
        obj_func_decr_count = obj_func_decr_count +1;
    else
        obj_func_decr_count = 0;        
    end
    if G_Z0 > G_Z0_old && G_Z0>0
        obj_func_incr_count = obj_func_incr_count +1;
    else
        obj_func_incr_count = 0;
    end
    

    % Print log or not
    if param.verbose && ((not(mod(iter, 500)) && iter>500)  || (not(mod(iter, 50)) && iter<=500 ))
        fprintf(...
            ['Iter: %i, Primal: %.2e, Dual: %.2e, Rho = %g, err = %.2e, ' ...
            'Tr = %g, Obj(X) = %.2e, Stop.Crit.: %.2e %.2e\n'], iter, ...
            res_primal, res_dual, param.rho, norm(A(Z0,M), 'fro')/norm(M), ...
            trace(Z0), G_Z0, p*eps_abs + eps_rel*max(Ax, Bz), ...
            n*eps_abs + eps_rel*Aty);
    end
    
    % Stopping criterion
    if res_primal <= p*eps_abs + eps_rel*max(Ax, Bz) && ...
            res_dual <= n*eps_abs + eps_rel*Aty
        fprintf('Terminated after convergence\n');
        break;
    end
    if (G_Z0 > param.Glimit  && obj_func_incr_count > 1000) || (G_Z0 < -param.Glimit  && obj_func_decr_count > 1000)
        G_Z0=sign(norm(Z0(Omega), 1) + real(signX'*Z0(not(Omega))))*Inf; 
        fprintf('Terminated early due to unbounded Obj. function\n');
        break;
    end
    if c>0 && G_Z0 > 0 && obj_func_incr_count > 1000 && iter > 2000
        fprintf('Terminated early due to increasing (+) Obj. function with c>0\n');
        break;
    end
    if c<=0 && G_Z0 < 0 && obj_func_decr_count > 1000 && iter > 2000
        fprintf('Terminated early due to decreasing (-) Obj. function with c<=0\n');
        break;
    end
    
    % Update rho
    if res_primal > param.mu*res_dual
        param.rho = param.rho*param.tau;
    elseif res_dual > param.mu*res_primal
        param.rho = param.rho/param.tau;
    end    
    
end
fprintf('Terminated after max. nb. of iterations\n');

end

function Y=A(X,M)
N = size(M,1);
L = size(X,2)/N;
    for k=L:-1:1
        for el=L:-1:1            
            Y(:,k,el) = sum((M'*X(((k-1)*N+1):(k*N),((el-1)*N+1):(el*N))).*(M.'), 2);
        end
    end
    Y = Y(:);
end

function X=At(Y,M)
N = size(M,1);
L = round(sqrt(length(Y)/size(M,2)));
Y = reshape(Y, size(M,2),L,L);
    for k=L:-1:1
        for el=L:-1:1
            X(((k-1)*N+1):(k*N),((el-1)*N+1):(el*N)) = M*diag(Y(:,k,el))*M';
        end
    end
end
